/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -c DBusNotify -p dbusnotify org.freedesktop.Notifications.xml
 *
 * qdbusxml2cpp is Copyright (C) 2016 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "dbusnotify.h"

/*
 * Implementation of interface class DBusNotify
 */

const QString NOTIFY_DBUS_NAME = "org.freedesktop.Notifications";
const QString NOTIFY_DBUS_PATH = "/org/freedesktop/Notifications";

DBusNotify::DBusNotify(QObject *parent)
    : QDBusAbstractInterface(NOTIFY_DBUS_NAME, NOTIFY_DBUS_PATH,
                             staticInterfaceName(), QDBusConnection::sessionBus(), parent)
{
    QDBusConnection::sessionBus().connect(this->service(), this->path(),
                                          "org.freedesktop.DBus.Properties", "PropertiesChanged", "sa{sv}as",
                                          this, SLOT(__propertyChanged__(QDBusMessage)));
}

DBusNotify::~DBusNotify()
{
    QDBusConnection::sessionBus().disconnect(service(), path(),
                                             "org.freedesktop.DBus.Properties", "PropertiesChanged", "sa{sv}as",
                                             this, SLOT(propertyChanged(QDBusMessage)));
}

