/****************************************************************************
** Meta object code from reading C++ file 'dbusnotify.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "src/dbusnotify.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusnotify.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10DBusNotifyE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusNotify::qt_create_metaobjectdata<qt_meta_tag_ZN10DBusNotifyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusNotify",
        "ActionInvoked",
        "",
        "in0",
        "in1",
        "NotificationClosed",
        "RecordAdded",
        "__propertyChanged__",
        "QDBusMessage",
        "msg",
        "ClearRecords",
        "QDBusPendingReply<>",
        "CloseNotification",
        "GetAllRecords",
        "QDBusPendingReply<QString>",
        "GetCapbilities",
        "QDBusPendingReply<QStringList>",
        "GetServerInformation",
        "QDBusPendingReply<QString,QString,QString,QString>",
        "QDBusReply<QString>",
        "QString&",
        "out1",
        "out2",
        "out3",
        "Notify",
        "QDBusPendingReply<uint>",
        "in2",
        "in3",
        "in4",
        "in5",
        "QVariantMap",
        "in6",
        "in7",
        "RemoveRecord"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ActionInvoked'
        QtMocHelpers::SignalData<void(uint, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'NotificationClosed'
        QtMocHelpers::SignalData<void(uint, uint)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 }, { QMetaType::UInt, 4 },
        }}),
        // Signal 'RecordAdded'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot '__propertyChanged__'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'ClearRecords'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'CloseNotification'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint)>(12, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'GetAllRecords'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(13, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'GetCapbilities'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(15, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'GetServerInformation'
        QtMocHelpers::SlotData<QDBusPendingReply<QString,QString,QString,QString>()>(17, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'GetServerInformation'
        QtMocHelpers::SlotData<QDBusReply<QString>(QString &, QString &, QString &)>(17, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 20, 22 }, { 0x80000000 | 20, 23 },
        }}),
        // Slot 'Notify'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &, uint, const QString &, const QString &, const QString &, const QStringList &, const QVariantMap &, int)>(24, 2, QMC::AccessPublic, 0x80000000 | 25, {{
            { QMetaType::QString, 3 }, { QMetaType::UInt, 4 }, { QMetaType::QString, 26 }, { QMetaType::QString, 27 },
            { QMetaType::QString, 28 }, { QMetaType::QStringList, 29 }, { 0x80000000 | 30, 31 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'RemoveRecord'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(33, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DBusNotify, qt_meta_tag_ZN10DBusNotifyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DBusNotify::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10DBusNotifyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10DBusNotifyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10DBusNotifyE_t>.metaTypes,
    nullptr
} };

void DBusNotify::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusNotify *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ActionInvoked((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->NotificationClosed((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 2: _t->RecordAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->__propertyChanged__((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 4: { QDBusPendingReply<> _r = _t->ClearRecords();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->CloseNotification((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QString> _r = _t->GetAllRecords();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QStringList> _r = _t->GetCapbilities();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QString,QString,QString,QString> _r = _t->GetServerInformation();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString,QString,QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusReply<QString> _r = _t->GetServerInformation((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<uint> _r = _t->Notify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->RemoveRecord((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusNotify::*)(uint , const QString & )>(_a, &DBusNotify::ActionInvoked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusNotify::*)(uint , uint )>(_a, &DBusNotify::NotificationClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusNotify::*)(const QString & )>(_a, &DBusNotify::RecordAdded, 2))
            return;
    }
}

const QMetaObject *DBusNotify::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusNotify::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10DBusNotifyE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusNotify::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void DBusNotify::ActionInvoked(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DBusNotify::NotificationClosed(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DBusNotify::RecordAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
